
# coding: utf-8


import cv2
import numpy as np

def digit2data(src, size=(8,8), flatten=False, border=0):
    h, w = src.shape[:2]
    square = src
    # 정사각형 형태로 만들기
    if h > w:
        pad = (h - w)//2
        square = np.zeros((h, h), dtype=np.uint8)
        square[:, pad:pad+w] = src
    elif w > h :
        pad = (w - h)//2
        square = np.zeros((w, w), dtype=np.uint8)
        square[pad:pad+h, :] = src
    px = np.zeros(size, np.uint8)
    content = size
    if border != 0 :
        content = (size[0] - (border*2), size[1] - (border*2))
        px[border:-border, border:-border] = cv2.resize(square, content, interpolation=cv2.INTER_AREA)
    else:
        px[:, :] = cv2.resize(square, content, interpolation=cv2.INTER_AREA)
    if flatten:
        px = px.reshape((1,-1))
    return px 

def digit_split(image, size=None, flatten=False, border=0, inverse=True):
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    _, th = cv2.threshold(gray, 127, 255, cv2.THRESH_BINARY_INV)
    (major, minor, _ )= cv2.__version__.split(".")
    if int(major) > 3 :
        contours, hirachy = cv2.findContours(th, cv2.RETR_EXTERNAL, 
                                        cv2.CHAIN_APPROX_SIMPLE)
    else :
        _, contours, hirachy = cv2.findContours(th, cv2.RETR_EXTERNAL, 
                                        cv2.CHAIN_APPROX_SIMPLE)
    digits = {}
    #numbers = []
    for c in contours:
        # 컨투어를 감싸는 외접 사각형으로 숫자 영역 좌표 구하기 ---⑦
        (x, y, w, h) = cv2.boundingRect(c)    
        # 외접 사각형의 크기가 너무 작은것은 제외 ---⑧
        if w >= 5 and h >= 25:
            # 숫자 영역만 roi로 확보하고 사각형 그리기 ---⑨
            roi = gray[y:y + h, x:x + w]
            #numbers.append(roi)
            digits[x] = roi
    numbers = [v for k, v in sorted(digits.items())] # 왼쪽 부터 순서대로 정렬
        
    if inverse:
        for i, n in enumerate(numbers):
            numbers[i] = 255- n  # 반전
        
    if size is None:
        return numbers
    else:
        for i, n in enumerate(numbers):
            numbers[i] = digit2data(n, size, flatten, border)
        return numbers



from .main import download, imshows, getImage
from .house import load_boston
from .img2data import digit_split
from .bbox import draw_bbox, get_iou

__all__ = ["download", 'imshows', 'load_boston', 'digit_split', 'draw_bbox', 'get_iou','getImage']
__version__ = '0.2.10'


import pandas as pd
from sklearn.utils import Bunch
import os

descr = ''

def load_boston():
  abs_path = os.path.abspath(__file__)
  path = os.path.dirname(abs_path)
  filename = "boston_house_prices.csv"
  df = pd.read_csv(os.path.join(path, 'resources', filename), header=0)
  with open(os.path.join(path, 'resources', 'descr.rst'), 'rt') as f:
    descr = f.read()
  bunch = Bunch(data=df.drop('MEDV', axis=1).values, 
                target=df['MEDV'].values,
                feature_names = df.columns[:-1].values,
                DESCR=descr,
                filename = filename,
                data_module='sklearn.dataset.data')
  return bunch






import math
import numpy as np
import matplotlib.pyplot as plt
import urllib.request, os
from urllib.parse import urlparse
import zipfile, tarfile, gzip


def imshows(imgs, rows=1, cols=None, bgr=True, figsize=None, axis=False):
  #모든 imgs는 zip 형태가 되게 강제한다.
  if isinstance(imgs, np.ndarray): #NumPy 1개
    size = 1
    imgs = zip([''], [imgs])
  elif isinstance(imgs, (list, tuple)): #List, Tuple
    size = len(imgs)
    imgs = zip(['' for _ in imgs], imgs)
  elif isinstance(imgs, dict): #Dict
    size = len(imgs)
    imgs = imgs.items()
  else:
    raise Exception(f'imshows:Not supported image type:{type(imgs)}')

  #행열 설정, 1행이면 n열, 행이 여러개면 열은 이미지 수/행으로 설정
  if rows == 1:
    cols = size  
  elif cols== None:
    cols = math.ceil(size/rows)

  #전체 이미지 표시
  if figsize:
    plt.figure(figsize=figsize)

  for i , (title, img) in enumerate(imgs, start=1):
    plt.subplot(rows, cols, i)
    plt.title(title)
    if img.ndim == 2:
        plt.imshow(img, cmap='gray')
    elif img.ndim == 3:
        if bgr:
            plt.imshow(img[:,:,::-1])
        else:
            plt.imshow(img)
    if axis == False:
      plt.xticks([]); plt.yticks([])
  plt.show();

def getImage(name, dir=None):
  url = f'https://github.com/dltpdn/download/blob/master/img/{name}?raw=true'
  download(url, dir)

def download(url, dir=None, file_name=None, extract=False):
  if file_name is None:
    parsed = urlparse(url)
    path = parsed.path

    if parsed.netloc == 'github.com':
      split_str = 'master/'
      if split_str in parsed.path:
        path = path.split(split_str, 1)[-1]  # 'master/' 이후 부분 추출
      if not dir:
        dir = os.path.dirname(path)

    if not dir:
      dir='.'

    if not file_name:
        file_name = os.path.basename(path)
  os.makedirs(dir, exist_ok=True)

  to=os.path.join(dir, file_name)
  try:
    urllib.request.urlretrieve(url, to)
  except:
    headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'}
    req = urllib.request.Request(url, headers=headers)
    with urllib.request.urlopen(req) as res:
      with open(to, 'wb') as outfile:
            outfile.write(res.read())
   
  print(f"{to} 파일이 다운로드되었습니다.")

  if extract:
    ext = os.path.splitext(to)[-1]

    if '.zip' == ext :
      with zipfile.ZipFile(to, 'r') as zip:
        ret = zip.extractall(dir)
        print(f'압축풀기:{to}, {ret}')
    elif '.tar' == ext or '.tgz' == ext:
      with tarfile.open(to) as tar:
        tar.extractall(dir)
        
  return to


from google.colab.patches import cv2_imshow
from google.colab import output
from IPython.display import display, Javascript, clear_output

def setMouseCallback(imgs, onmouse=None):
  if onmouse:
    js_code = Javascript('''
      var dragging = false;
      function callback(e, event_name){
          var x = e.offsetX;
          var y = e.offsetY;
          var event=0
          var flag=0;

          if(event_name =='mousedown'){
            if(e.button==0){
              event=1;

              dragging=true;
            }else if(e.button==2){
              event=2;
              flag+=2
            }
          }else if(event_name=='mouseup'){
            if(e.button==0){
              event=4;

              if(dragging) dragging=false;
            }else if(e.button==2){
              event=5;

            }
          }else if(event_name=='mousemove'){
            event=0;
            if(dragging){
              flag+=1
            }
            console.log(e.button)
          }

          if(e.altKey){
            flag+=32;
          }
          if(e.ctrlKey){
            flag+=8;
          }
          if(e.shiftKey){
            flag+=16
          }
          google.colab.kernel.invokeFunction('notebook.onmouse', [event, x, y, flag, null], {});
      }
      window.setTimeout(()=>{
        var img = document.querySelector('#output-area img');
        img.draggable=false;
        img.addEventListener('mousedown',(e)=>{
          callback(e, 'mousedown')
        });
        img.addEventListener('contextmenu', (e)=>{e.preventDefault();})
        img.addEventListener('mouseup', (e)=>{
          callback(e, 'mouseup')
        });
        img.addEventListener('mousemove',(e)=>{
          console.log('move')
          callback(e, 'mousemove')
        });
      }, 0);
    ''')
    cv2_imshow(imgs)
    output.register_callback('notebook.onmouse', onmouse)
    display(js_code)
  else:
    print('no mouse event!!')



def selectROI(img, callback):
    cv2_imshow(img) #출력 영역에 이미지 표시
    if callback==None:
      print('no callback registered.')
      return
      
    output.register_callback('notebook.onmouse', callback) #calab output으로 콜랩 등록
    
    js_code = """
    window.setTimeout(() => { //렌더링까지 시간이 걸려서 10ms 대기 후 실행
        let img = document.querySelector('#output-area img');//cv_imshow()를 통해 출력되는 이미지 요소
        if (!img) return;

        img.draggable = false;
        img.style.cursor = 'crosshair'; 
        let canvas = document.createElement('canvas');
        let ctx = canvas.getContext('2d');
        let rect = {};
        let isDrawing = false;
        let boxColor = 'blue'
        canvas.width = img.width;
        canvas.height = img.height;
        canvas.style.position = 'absolute';
        canvas.style.top = img.offsetTop + 'px';
        canvas.style.left = img.offsetLeft + 'px';
        canvas.style.pointerEvents = 'none';
        
        //canvas.style.backgroundColor = 'rgba(0, 0, 0, 0.3)'; // 반투명 배경 적용
        
        document.body.appendChild(canvas);

        function startDraw(event) {
            console.log(event);
            rect.startX = event.offsetX;
            rect.startY = event.offsetY;
            isDrawing = true;
        }

        function draw(event) {
            if (!isDrawing) return;
            rect.w = event.offsetX - rect.startX;
            rect.h = event.offsetY - rect.startY;
            ctx.clearRect(0, 0, canvas.width, canvas.height);
            ctx.strokeStyle = boxColor;
            ctx.lineWidth = 2;
            ctx.strokeRect(rect.startX, rect.startY, rect.w, rect.h);
        }

        function stopDraw(event) {
            if (isDrawing){
              boxColor= 'red'
              draw(event);
              boxColor= 'blue'
              isDrawing = false;
              
              //document.body.removeChild(canvas);
              google.colab.kernel.invokeFunction('notebook.onmouse', [rect.startX, rect.startY, rect.w, rect.h], {});
            }
            
        }

        img.addEventListener('mousedown', startDraw);
        img.addEventListener('mousemove', draw);
        img.addEventListener('mouseup', stopDraw);
    }, 10);
    """
    display(Javascript(js_code))


import cv2
import numpy as np

def draw_bbox(img, x1, y1, x2, y2, text=None, txt_color=(255,255,255), bb_color=None, thickness=2, font_size=0.7):
  draw = img.copy()
  font = cv2.FONT_HERSHEY_DUPLEX
  if bb_color is None:
    bb_color = np.random.randint(0, 200, 3).tolist()
  #bbox 그리기
  cv2.rectangle(draw, (x1, y1), (x2, y2), bb_color, thickness)
  if text:
    #text 크기 구하기
    txt_w, txt_h = cv2.getTextSize(text, font, font_size, 1)[0]
    #text 영역 그리기
    cv2.rectangle(draw, (x1, y1), (x1+txt_w+4, y1+txt_h+4), bb_color, thickness=-1)
    #text 출력
    cv2.putText(draw, text, (x1, y1+txt_h), font, font_size, color=txt_color)
  return draw


def get_iou(pred, gt):
  x1 = np.maximum(pred[0], gt[0]) #좌상단 x
  y1 = np.maximum(pred[1], gt[1]) #좌상단 y
  x2 = np.minimum(pred[2], gt[2]) #우하단 x
  y2 = np.minimum(pred[3], gt[3]) #우하단 y

  pred_area = (pred[2] - pred[0]) * (pred[3] - pred[1])
  gt_area = (gt[2] - gt[0]) * (gt[3] - gt[1])
  intersection = np.maximum(x2 - x1, 0) * np.maximum(y2 - y1, 0)
  union = pred_area + gt_area - intersection
  iou = intersection / union
  return iou